#ifndef BOARD_BT6703_H
#define BOARD_BT6703_H

#define SERVICE_RS232_UART_TABLE(X)     \
X( SERVICE_TXDINT,   GPIOD,   5, AF_USART2) \
X( SERVICE_RXDINT,   GPIOD,   6, AF_USART2)

#define RS232_UART_TABLE(X)     \
X( TXDINT,   GPIOA,   9, AF_USART1) \
X( RXDINT,   GPIOA,   10, AF_USART1)

#define SPI2_TABLE(X) \
X( SPI2_MISO,   GPIOB,  14, AF_SPI2) \
X( SPI2_MOSI,   GPIOB,  15, AF_SPI2) \
X( SPI2_SCK,    GPIOB,  10, AF_SPI2) \
X( SPI2_NSS,    GPIOB,  9,  GPIO_OUT | GPIO_SET)

#define MCU_PERIPHERALS(X)  \
SERVICE_RS232_UART_TABLE(X)\
RS232_UART_TABLE(X) \
SPI2_TABLE(X)

#define DI_TABLE(X)                  \
X( DIN1,       GPIOA,   0, GPIO_IN)

#define RELAYS(X) \
X( DOUT1,      GPIOE,  4, GPIO_OUT)\
X( DOUT2,      GPIOE,  5, GPIO_OUT)

#define LEDS(X) \
X( LED_INIT_R,  GPIOA,  4,  GPIO_OUT | GPIO_INV) \
X( LED_INIT_G,  GPIOA,  3,  GPIO_OUT | GPIO_INV) \
X( LED_MAJOR_R, GPIOA,  5,  GPIO_OUT | GPIO_INV) \
X( LED_MINOR_G, GPIOE,  11, GPIO_OUT | GPIO_INV) \
X( LED_MINOR_R, GPIOE,  12, GPIO_OUT | GPIO_INV)

#define JUMPERS(X) \
X( MODE_JUMPER, GPIOE,  6,  GPIO_IN_PU | GPIO_INV)

#define BUTTONS(X) \
X( KEY_DEF,     GPIOE,  7,  GPIO_IN )//| GPIO_INV

#define WDG_PIN(X) \
X( _WDG,     GPIOE,   15, GPIO_OUT)

#define GPIO_TABLE(X)   \
MCU_PERIPHERALS(X)  \
DI_TABLE(X)         \
RELAYS(X) \
LEDS(X)             \
JUMPERS(X) \
BUTTONS(X)      \
WDG_PIN(X)

/**
  * @brief  Маркер ревизии платы для валидации *.bin файла
  *
  */
#define HW_REV                          "BT-6703"

/**
  * @brief  Версия прошивки
  */
#define VERSION                         "1.0"

#define RTC_ENABLE                  //RTC clock
#define LED_ENABLE                  //LED индикация
#define BUTTON_ENABLE               //Кнопки
#define INOUTS_ENABLE               //Входы/выходы
#define NET_ENABLE                  //Ethernet сеть
#define SSL_SERVER_ENABLE           //HTTPS сервер
#define RADIUS_SERVER_ENABLE        //RADIUS сервер
#define SNMP_ENABLE                 //SNMP протокол
#define UPS_ENABLE                  //UPS
//#define DEBUG_FREERTOS            //Мониторинг FreeRtos

#define SETTINGS_TABLE \
XSETTING( SNMP_t,	sSnmp, SETTINGS_SetSnmpDef, ALL_DEFAULT ) \
XSETTING( AUTH_t,	sAuth[MAX_WEB_USERS], SETTINGS_SetServiceDef, PART_DEFAULT_2 ) \
XSETTING( SNTP_t,	sSNTP, SETTINGS_SetSntpDef, ALL_DEFAULT ) \
XSETTING( INOUTS_t,	sInOuts, SETTINGS_SetINOUTSDef, ALL_DEFAULT ) \
XSETTING( UPS_Setting_t, UPS_Setting, SETTINGS_SetUPSSettingsDef, ALL_DEFAULT ) \
XSETTING( ALARM_MNGR_t, sAlarmManager, SETTINGS_SetAlarmManagerDef, ALL_DEFAULT ) \
XSETTING( RADIUS_t, sRADIUS, SETTINGS_SetRADIUSDef, PART_DEFAULT_1 ) \
XSETTING( char, our_srv_crt[1705], SETTINGS_SetSSLcrtDef, PART_DEFAULT_1 ) \

#define SNMP_DEV_ROOT_OID       6

#define MAX_IRQ_HANDLERS 4

#define WEB_SETTINGS_TAGS_TABLE \
  XJSON_SETTINGS_TAG("read_community",     GetReadCommunity) \
  XJSON_SETTINGS_TAG("write_community",    GetWriteCommunity) \
  XJSON_SETTINGS_TAG("managerIP",          GetManagerIp) \
  XJSON_SETTINGS_TAG("managerIP2",         GetManagerIp2) \
  XJSON_SETTINGS_TAG("managerIP3",         GetManagerIp3) \
  XJSON_SETTINGS_TAG("managerIP4",         GetManagerIp4) \
  XJSON_SETTINGS_TAG("managerIP5",         GetManagerIp5) \
  XJSON_SETTINGS_TAG("di1",                GetDIN1TypeActStr) \
  XJSON_SETTINGS_TAG("ro1",                GetRO0TypeActStr) \
  XJSON_SETTINGS_TAG("ro2",                GetRO1TypeActStr) \
  XJSON_SETTINGS_TAG("ipaddr",             GetIpStr) \
  XJSON_SETTINGS_TAG("gw",                 GetGatewayStr) \
  XJSON_SETTINGS_TAG("mask",               GetMaskStr) \
  XJSON_SETTINGS_TAG("dhcp",               GetDhcpStateStr) \
  XJSON_SETTINGS_TAG("swauth",             GetAuthEnableStateStr) \
  XJSON_SETTINGS_TAG("rs_server",          GetRDSIpStr) \
  XJSON_SETTINGS_TAG("rs_port",            GetRDSPortStr) \
  XJSON_SETTINGS_TAG("rs_pwd",             GetRDSPasswordkStr) \
  XJSON_SETTINGS_TAG("rs_key",             GetRDSKeyAccesstStr) \
  XJSON_SETTINGS_TAG("rs_enabled",         GetRDSEnableStateStr) \
  XJSON_SETTINGS_TAG("date",               GetDateStr) \
  XJSON_SETTINGS_TAG("time",               GetTimeStr) \
  XJSON_SETTINGS_TAG("ntp",                GetSntpStateStr) \
  XJSON_SETTINGS_TAG("ntpservip",          GetSntpServerIpStr) \
  XJSON_SETTINGS_TAG("utc",                GetSntpTimeZoneStr) \
  XJSON_SETTINGS_TAG("utm",                GetUnixTimeStr) \
  XJSON_SETTINGS_TAG("lastsynctime",       GetSntpLastDataStr) \
  XJSON_SETTINGS_TAG("ups_cell_min",       GetUPSVoltCellMinStr) \
  XJSON_SETTINGS_TAG("ups_cell_max",       GetUPSVoltCellMaxStr) \
  XJSON_SETTINGS_TAG("temp_high",          GetTemperatureAlarmHighRangeStr) \
  XJSON_SETTINGS_TAG("temp_low",           GetTemperatureAlarmLowRangeStr) \
  XJSON_SETTINGS_TAG("temp_hist",          GetTemperatureAlarmHisteStr) \
  XJSON_SETTINGS_TAG("loadvolt_high",      GetLoadAlarmHighRangeStr) \
  XJSON_SETTINGS_TAG("loadvolt_hist",      GetLoadAlarmHisteStr) \
  XJSON_SETTINGS_TAG("netsettings_changed",GetWebReinitFlag) \

  #define WEB_PARAMS_TAGS_TABLE \
  XJSON_PARAMS_TAG("AC",                 GetInputVoltageStr) \
  XJSON_PARAMS_TAG("DC",                 GetOutputVoltageStr) \
  XJSON_PARAMS_TAG("in_freq",            GetInputFreqStr) \
  XJSON_PARAMS_TAG("out_freq",           GetOutputFreqStr) \
  XJSON_PARAMS_TAG("pwr",                GetPowerStr) \
  XJSON_PARAMS_TAG("bat_rem_cap",        GetBatCapacityStr) \
  XJSON_PARAMS_TAG("inner_temp",         GetInternalTempStr) \
  XJSON_PARAMS_TAG("bat_time_left",      GetRuntimeStr) \
  XJSON_PARAMS_TAG("di1",                GetDIN0StatusStr) \
  XJSON_PARAMS_TAG("ro1",                GetDOUT0StatusStr) \
  XJSON_PARAMS_TAG("ro2",                GetDOUT1StatusStr) \
  XJSON_PARAMS_TAG("m_alarm",            GetAlarmMonitorStr) \
  XJSON_PARAMS_TAG("u_alarm",            GetAlarmStr) \
  XJSON_PARAMS_TAG("netsettings_changed",GetWebReinitFlag) \

#define TRAPS_TABLE \
  XTRAP(FW_VERSION_UPDATE,                 1,      true,      GetVersionStr) \
  XTRAP(FW_VERSION_UPDATED,                1,      true,      GetVersionStr) \
  XTRAP(DEVICE_RESTORED,                   1,      true,      GetVersionStr) \
  XTRAP(DI0_NORM,                          9,      true,      GetDIN0StatusStr) \
  XTRAP(DI0_ALARM,                         9,      true,      GetDIN0StatusStr) \
  XTRAP(DO0_TOGGLED,                       5,      true,      GetDOUT0StatusStr) \
  XTRAP(DO1_TOGGLED,                       6,      true,      GetDOUT1StatusStr) \
  XTRAP(BATTERY_HIGH_TEMPERATURE_NORM,     10,     true,      GetInternalTempStr) \
  XTRAP(BATTERY_HIGH_TEMPERATURE_ALARM,    10,     true,      GetInternalTempStr) \
  XTRAP(LINE_ALARM,                        12,     true,      GetInputVoltageStr) \
  XTRAP(LINE_NORM,                         12,     true,      GetInputVoltageStr) \
  XTRAP(LOW_BAT_ALARM,                     15,     true,      GetBatCapacityStr) \
  XTRAP(LOW_BAT_NORM,                      15,     true,      GetBatCapacityStr) \
  XTRAP(POWER_ALARM,                       14,     true,      GetPowerStr) \
  XTRAP(POWER_NORM,                        14,     true,      GetPowerStr) \
  XTRAP(CONNECT_MONITOR_ALARM,             17,     true,      GetConnectMonitorStr) \
  XTRAP(CONNECT_MONITOR_NORM,              17,     true,      GetConnectMonitorStr) \
  XTRAP(BATTERY_CONNECT_ALARM,             15,     true,      GetBatCapacityStr) \
  XTRAP(BATTERY_CONNECT_NORM,              15,     true,      GetBatCapacityStr) \
  XTRAP(BATTERY_LOW_TEMPERATURE_NORM,      10,     true,      GetInternalTempStr) \
  XTRAP(BATTERY_LOW_TEMPERATURE_ALARM,     10,     true,      GetInternalTempStr) \

#if 0
  const char* logsStrShortRu[] =
{
	"Перезагрузка контроллера",
    "Сброс настроек",
    "Обновление ПО",
    "Включение",
    "Смена пароля",
    "Сохранение настроек",
    "Авторизация",
    "Тест ИБП",
    "Откл. нагрузки ИБП",
    "Авария дискр. входа 1",
    "Состояние выхода 1",
    "Состояние выхода 2",
    "Высокая температура",
    "Низкая температура",
    "Авария вх. напряжения",
    "Низкий заряд АКБ",
    "Авария нагрузки",
    "Авария связи с ИБП",
    "Авария отключения АКБ",
};
#endif

#endif /* BOARD_PRS_H */
